/*
 * DCUPComponentManagerConnector.java
 *
 * Created on 5. duben 2002, 13:50
 */

package SOFA.Connector.Boot;

import SOFA.Connector.*;

/**
 *
 * @author  ghort
 * @version 
 */
public class DCUPComponentManagerConnector implements SOFA.Connector.ECG.Types.CSProcCall {

	/** Creates new DCUPComponentManagerConnector */
	public DCUPComponentManagerConnector() {
	}

	static public RoleBase createConnector(SOFA.Connector.ECG.DeploymentDescriptor dd) throws ConnectorException {
		RoleBase role=null;
		if (dd.unit.equals("Client")) {
			role=new SOFA.Connector.Boot.DCUPComponentManagerCRole();
		} else if (dd.unit.equals("Server")) {
			role=new SOFA.Connector.Boot.DCUPComponentManagerSRole();
			role.setDeploymentDescriptor(dd);
			
			Linkable skelRMI;
			String rmiName;
			try {
				rmiName="rmi://"+java.net.InetAddress.getLocalHost().getHostAddress()+":1099/SOFA/Connectors/Boot/DCUPComponentManagerRMISkeleton/"+new java.rmi.server.UID();
				skelRMI=new SOFA.Connector.Boot.DCUPComponentManagerRMISkeleton(rmiName);
			} catch (Exception e) {
				throw new ConnectorException("Can't instantiate DCUPComponentManagerRMISkeleton.",e);
			}
			skelRMI.link(role);

			Reference ref=new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DCUPComponentManagerConnector",
					new TaggedProfile[] { new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName) } );
			role.setSOFAReference(ref);
		} else {
			throw new ConnectorException("Unknown unit '"+dd.unit+"'.");
		}
		return role;
	}
  
  /** Creates server side of connector to CM 
    * @param ddock     CM
    * @param name      instance id  of the CM
    * @return          server side of connector
    * @author Petr Hnetynka
    */
  public static RoleBase createSrv(SOFA.Component.ComponentManager cm, String id) throws ConnectorException {
    RoleBase role = new SOFA.Connector.Boot.DCUPComponentManagerSRole();
    SOFA.Connector.ECG.DeploymentDescriptor cDD = new SOFA.Connector.ECG.DeploymentDescriptor();
    cDD.unit="Server";
    role.setDeploymentDescriptor(cDD);
    
    Linkable skelRMI;
    String rmiName;
    String rmihost = System.getProperty("sofa.rmihost", "localhost");
    String rmiport = System.getProperty("sofa.rmiport", "1099");
    try {
      rmiName="rmi://"+rmihost+":"+rmiport+"/SOFA/Connectors/Boot/DCUPComponentManagerRMISkeleton/"+id;
      skelRMI = new SOFA.Connector.Boot.DCUPComponentManagerRMISkeleton(id);
    } catch (Exception e) {
      throw new ConnectorException("Can't instantiate DCUPComponentManagerRMISkeleton.",e);
    }
    skelRMI.link(role);

    // ???
    Reference ref=new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DCUPComponentManagerConnector",
    new TaggedProfile[] { new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName) } );
    role.setSOFAReference(ref);
    role.link(cm);
    return role;
  }
  
  /** Creates client side of connector to CM 
    * @param   ref    reference to CM
    * @return         client side of connector
    * @author Petr Hnetynka
    */
  public static RoleBase createClt(SOFA.Connector.Reference ref) throws ConnectorException {
    RoleBase role = new SOFA.Connector.Boot.DCUPComponentManagerCRole();
    try {
      role.link(ref);
    } catch (SOFA.Connector.LinkException e) {
      throw new ConnectorException("Can't link connector to CM.", e);
    }
    return role;
  }

}
