/*
 * DataStreamLocalReceiverGen.java
 *
 * Created on 13. duben 2002, 21:08
 */

package SOFA.Connector.EEG.EEM.Elements;

import SOFA.SOFAnode.Made.TIR.*;
import SOFA.SOFAnode.Made.TIR.Access.TIRAccessMethods;
import SOFA.Connector.EEG.CodeWriter.*;
import SOFA.Connector.EEG.EEM.Actions.*;

/**
 *
 * @author  ghort
 * @version 
 */
public class DataStreamLocalReceiverGen extends JImplWriterDirectorBase {

	protected String is, os, bh, isPackage, osPackage;

	protected static final int pullMode=0;
	protected static final int pushMode=1;
	
	protected int sMode, rMode;

	/** Creates new DataStreamLocalReceiverGen */
	public DataStreamLocalReceiverGen(String provIfaceCDL, SOFA.Connector.Property[] params) throws JWriterDirectorException {
		super(provIfaceCDL,params);
		isPackage=SOFA.Connector.Property.findFirst(params,"isPackage");
		osPackage=SOFA.Connector.Property.findFirst(params,"osPackage");
		is=isPackage+".CNInterface";
		os=osPackage+".CNInterface";
		bh="SOFA.Connector.EEG.Types.DataStream.bytesHolder";

		String mode=SOFA.Connector.Property.findFirst(params,"senderMode");
		if (mode!=null) {
			if (mode.equals("push")) {
				sMode=pushMode;
			} else if (mode.equals("pull")) {
				sMode=pullMode;
			} else {
				throw new JWriterDirectorException("Parameter senderMode is incorrect '"+mode+"'.");
			}
		} else {
			throw new JWriterDirectorException("Parameter senderMode is not specified");
		}

		mode=SOFA.Connector.Property.findFirst(params,"receiverMode");
		if (mode!=null) {
			if (mode.equals("push")) {
				rMode=pushMode;
			} else if (mode.equals("pull")) {
				rMode=pullMode;
			} else {
				throw new JWriterDirectorException("Parameter receiverMode is incorrect '"+mode+"'.");
			}
		} else {
			throw new JWriterDirectorException("Parameter receiverMode is not specified");
		}

		if (sMode==pullMode && rMode==pushMode) {
			throw new JWriterDirectorException("Mode sender-push/receiver-pull is not supported.");
		}
	}

	public String[] getImplements() {
		if (rMode==pullMode) {
      if (sMode==pullMode) {
  			return new String[] {"SOFA.Connector.EEG.Types.Receiver","SOFA.Connector.EEG.EEM.Elements.DataStreamLocalNotifiable",is};
      } else {
  			return new String[] {"SOFA.Connector.EEG.Types.Receiver",is};
      }
		} else {
			return new String[] {"SOFA.Connector.EEG.Types.Receiver","SOFA.Connector.Linkable",os};
		}
	}
	
	public String getClassBody() throws JWriterDirectorException {
		String methods;
		if (sMode==pushMode && rMode==pushMode) {
			methods=
				"\tpublic void write(byte[] buffer, int len, String thr_id) {\n"+
				"\t\tclient.write(buffer,len,thr_id);\n"+
				"\t}\n\n";
		} else if (sMode==pullMode && rMode==pullMode) {
			methods=
				"\tprotected java.lang.reflect.Method senderRead;\n"+
				"\tprotected Object senderReadParams[]=new Object[3];\n\n"+
        "\tpublic void linkNotification(Object sender) throws SOFA.Connector.LinkException {\n"+
        "\t\tthis.sender=sender;\n"+
        "\t\ttry {\n"+
        "\t\t\tsenderRead=sender.getClass().getMethod(\"read\",new Class[] {"+bh+".class, Integer.TYPE, String.class});\n"+
        "\t\t} catch (NoSuchMethodException e) {\n"+
        "\t\t\tthrow new SOFA.Connector.LinkException(\"Cannot create dynamic local binding\",e);\n"+
        "\t\t}\n"+
        "\t}\n\n"+
				"\tpublic int read("+bh+" bufferHolder, int len, String thr_id) {\n"+
        "\t\tsenderReadParams[0]=bufferHolder;\n"+
        "\t\tsenderReadParams[1]=new Integer(len);\n"+
        "\t\tsenderReadParams[2]=thr_id;\n"+
        "\t\ttry {\n"+
				"\t\t\treturn senderRead.invoke(sender,senderReadParams).intValue();\n"+
        "\t\t} catch (IllegalAccessException _e) {\n"+
        "\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"IllegalAccessException\", _e);\n"+
        "\t\t} catch (java.lang.reflect.InvocationTargetException _e) {\n"+
        "\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"InvocationTargetException\", _e);\n"+
        "\t\t}\n"+
				"\t}\n\n";
		} else if (sMode==pushMode && rMode==pullMode) {
			String outClass=SOFA.Connector.Property.findFirst(params,"outClass");
			final int queueSize=16384;
			methods=
				"\tprotected java.nio.ByteBuffer queue;\n"+
				"\tprotected int queueStart, queueEnd;\n"+
				"\t"+outClass+"() {\n"+
				"\t\tqueue=java.nio.ByteBuffer.allocateDirect("+queueSize+");\n"+
				"\t\tqueueStart=queueEnd=0;\n"+
				"\t}\n\n"+
				"\tpublic synchronized void write(byte[] buffer, int len, String thr_id) {\n"+
				"\t\tint rem=queueStart-queueEnd-1;\n"+
				"\t\tif (rem<0) {\n"+
				"\t\t\trem+="+queueSize+";\n"+
				"\t\t}\n"+
				"\t\twhile (rem<len) {\n"+
				"\t\t\ttry {\n"+
				"\t\t\t\twait();\n"+
				"\t\t\t} catch (InterruptedException e) {\n"+
				"\t\t\t}\n"+
				"\t\t\trem=queueStart-queueEnd-1;\n"+
				"\t\t\tif (rem<0) {\n"+
				"\t\t\t\trem+="+queueSize+";\n"+
				"\t\t\t}\n"+
				"\t\t}\n"+
				"\t\ttry {\n"+
				"\t\t\tqueue.position(queueEnd);\n"+
				"\t\t\tif (queueEnd<queueStart) {\n"+
				"\t\t\t\tqueue.put(buffer,0,len);\n"+
				"\t\t\t\tqueueEnd+=len\n"+
				"\t\t\t} else {\n"+
				"\t\t\t\tint partLen="+queueSize+"-queueEnd;\n"+
				"\t\t\t\tqueue.put(buffer,0,partLen);\n"+
				"\t\t\t\tqueue.position(0);\n"+
				"\t\t\t\tqueue.put(buffer,partLen,len-partLen);\n"+
				"\t\t\t\tqueueEnd=len-partLen\n"+
				"\t\t\t}\n"+
				"\t\t\tnotifyAll();\n"+
				"\t\t} catch (BufferOverflowException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t} catch (IndexOutOfBoundsException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t} catch (ReadOnlyBufferException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t}\n"+
				"\t}\n\n"+
				"\tpublic synchronized int read("+bh+" bufferHolder, int len, String thr_id) {\n"+
				"\t\tbyte[] buffer=bufferHolder.value;\n"+
				"\t\tint rem=queueEnd-queueStart;\n"+
				"\t\tif (rem<0) {\n"+
				"\t\t\trem+="+queueSize+";\n"+
				"\t\t}\n"+
				"\t\twhile (rem==0) {\n"+
				"\t\t\ttry {\n"+
				"\t\t\t\twait();\n"+
				"\t\t\t} catch (InterruptedException e) {\n"+
				"\t\t\t}\n"+
				"\t\t\trem=queueEnd-queueStart;\n"+
				"\t\t\tif (rem<0) {\n"+
				"\t\t\t\trem+="+queueSize+";\n"+
				"\t\t\t}\n"+
				"\t\t}\n"+
				"\t\tif (rem<len) {\n"+
				"\t\t\trem=len;\n"+
				"\t\t}\n"+
				"\t\ttry {\n"+
				"\t\t\tqueue.position(queueStart);\n"+
				"\t\t\tif (queueStart+rem<="+queueSize+") {\n"+
				"\t\t\t\tqueue.get(buffer,0,rem);\n"+
				"\t\t\t\tqueueStart+=rem\n"+
				"\t\t\t} else {\n"+
				"\t\t\t\tint partLen="+queueSize+"-queueStart;\n"+
				"\t\t\t\tqueue.get(buffer,0,partLen);\n"+
				"\t\t\t\tqueue.position(0);\n"+
				"\t\t\t\tqueue.put(buffer,partLen,rem-partLen);\n"+
				"\t\t\t\tqueueStart=rem-partLen\n"+
				"\t\t\t}\n"+
				"\t\t\tnotifyAll();\n"+
				"\t\t\treturn rem;\n"+
				"\t\t} catch (BufferOverflowException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t} catch (IndexOutOfBoundsException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t} catch (ReadOnlyBufferException e) {\n"+
				"\t\t\tthrow new SOFA.Connector.ConnectorTransportException(\"Unexpected exception caught.\",e);\n"+
				"\t\t}\n"+
				"\t}\n\n";
		} else {
			throw new JWriterDirectorException("Mode sender-push/receiver-pull is not supported.");
		}

		return managerTieImpl() + "\n" +
			(sMode==pullMode && rMode==pullMode ? "\tprotected Object sender;\n" : "")+
			"\tprotected "+os+" client;\n\n"+
			"\tpublic void link(Object target) {\n"+
			"\t\tclient=("+os+")target;\n"+
			"\t}\n\n"+
			"\tpublic SOFA.Connector.TaggedProfile[] getProfiles() {\n"+
			"\t\treturn new SOFA.Connector.TaggedProfile[] { new SOFA.Connector.ECG.Profiles.LocalProfile(this) };\n"+
			"\t}\n\n"+methods+"\n\n"+
                        "\tpublic org.objectweb.fractal.api.Type getFcItfType() { return null; }\n"+
                        "\tpublic boolean isFcInternalItf() { return false; }\n"+
                        "\tpublic String getFcItfName() { throw new SOFA.Connector.ConnectorTransportException(\"Interconection via EventPassing connector - getFcItfName is not supported.\"); }\n"+
                        "\tpublic org.objectweb.fractal.api.Component getFcItfOwner() { throw new SOFA.Connector.ConnectorTransportException(\"Interconection via EventPassing connector - getFcItfOwner is not supported.\"); }\n\n";
	}
	
	public String getMethodDef(OperationDef mt) throws JWriterDirectorException {
		return null;
	}

	public boolean defaultImplements() {
		return false;
	}
}
