/**
 * InOut2ClientConnector.java is a part of the SOFA project.
 * This file was created by pepan on 16.4.2003.
 */
package SOFA.SOFAnode.InOut.Connector;

import SOFA.Connector.*;
import SOFA.SOFAnode.InOut.InOut2Client;
import SOFA.SOFAnode.TR.TR2InOut;
import SOFA.Util.VMProperties;

/** Contains methods for creating a connector between the
 * {@link SOFA.SOFAnode.InOut.Impl.Browser client implementation} and the {@link InOut2Client} interface.
 *
 * @author Petr Panuska
 */
public class InOut2ClientConnector {

  /**
   * Creates the server side of a connector between {@link SOFA.SOFAnode.InOut.Impl.Browser Client}
   * and {@link SOFA.SOFAnode.InOut.Impl.InOutImpl InOut}. The connector is defined in
   * the {@link TR2InOut} interface.
   * @param inOut InOut which is going to be connectable.
   * @return the server side of the connector.
   */
  public static RoleBase createSrv (InOut2Client inOut) throws ConnectorException {
    RoleBase role = new InOut2ClientSRole();
    SOFA.Connector.ECG.DeploymentDescriptor cDD = new SOFA.Connector.ECG.DeploymentDescriptor();
    cDD.unit = "Server";
    role.setDeploymentDescriptor(cDD);

    Linkable skelRMI;
    String rmiName;
    String rmihost = System.getProperty(VMProperties.RMI_HOST, "localhost");
    String rmiport = System.getProperty(VMProperties.RMI_PORT, "1099");
    String inOutName = System.getProperty(VMProperties.TR_NAME, "localInOut");
    inOutName = "/SOFA/SOFAnode/InOut/Connector/InOut2ClientRMISkel/" + inOutName;
    rmiName = "//" + rmihost + ":" + rmiport + inOutName;
    try {
      skelRMI = new InOut2ClientRMISkel(inOutName);
    } catch (Exception e) {
      throw new ConnectorException("Can't instantiate InOut2ClientRMISkel.\n" + rmiName + "\n", e);
    }
    skelRMI.link(role);

    // ???
    Reference ref = new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DeplDockConnector",
      new TaggedProfile[]{new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName)});
    role.setSOFAReference(ref);
    role.link(inOut);
    return role;
  }

  /**
   * Creates the client side of the connector.
   * @param ref the reference to the client.
   * @return the client side of the connector.
   */
  public static RoleBase createClt (Reference ref) throws ConnectorException {
    RoleBase role = new SOFA.SOFAnode.InOut.Connector.InOut2ClientCRole();
    try {
      role.link(ref);
    } catch (LinkException e) {
      throw new ConnectorException("Can't link connector to InOut.\n" + ref + "\n", e);
    }
    return role;
  }

}
