/* $Id: CGFileAccess.java,v 1.2 2003/02/27 19:27:37 hnetynka Exp $ */
package SOFA.SOFAnode.Made.CodeGen;

import java.io.File;
import java.io.IOException;

/** Methods for accessing files and directories.
  *
  * @author Petr Hnetynka
  */
public class CGFileAccess {
  
  File _root;
  boolean rewrite;
  
  /** Constructor.
    * @param rootDir   root directory of created files
    * @param rewrite   if already existing files with the same name can be rewritten
    */
  public CGFileAccess(String rootDir, boolean rewrite) throws IOException {
    _root = new File(rootDir);
    if (! _root.isDirectory()) {  // given path is not directory
      throw new IOException(rootDir+" "+Res.res.getString("MSG_IsNotDir"));
    }
    this.rewrite = rewrite;
  }

  /** Returns root of created files */
  public File root() {
    return _root;
  }
  

  /** Creates new writer with given name relatively to the root
    *
    * @param name   name of created file
    * @exception IOException  if file cannot be created 
    */
  public CGFileWriter newCGWriter(String name) throws IOException {
    File path = new File(_root, name);
    if (path.exists()) {
      if (path.isDirectory())
        throw new IOException(path+" "+Res.res.getString("MSG_IsDir"));
      if (!rewrite)
        throw new IOException(path+" "+Res.res.getString("MSG_Exists"));
    }
    File dir = path.getParentFile();
    if (dir != null) {
      if (! dir.exists()) {
        if (! dir.mkdirs())
          throw new IOException(path+" "+Res.res.getString("MSG_CantBeCreated"));
      }
    }
    return new CGFileWriter(path);
  }

  /** Tests if file with given name (relatively to the root) exists.
    * 
    * @param name name of the file
    * @return <tt>true</tt> if file exists
    * @throws IOException IO exception
    */
  public boolean exists(String name) throws IOException {
    File path = new File(_root, name);
    return path.exists();
  }

  /** From CDL absolute name creates path. 
    *
    * @param abs         absolute name
    * @param shortName   short name (generated name can be different from abs name (reserved names))
    * @param extension   if add ".java" extension
    */
  public static String absToPath(SOFA.SOFAnode.Made.TIR.AbsoluteName abs, String shortName, boolean extension) throws java.rmi.RemoteException {
    StringBuffer sb = new StringBuffer();
    for (int i=0;i<abs.size()-1;i++) {
      sb.append(abs.elementAt(i));
//      if (i < abs.size()-1)
        sb.append(File.separator);
    }
    sb.append(shortName);
    if (extension)
      sb.append(".java");
    return sb.toString();
  }

  /** From CDL absolute name creates package. 
    *
    * @param abs         absolute name
    */
  public static String absToPkg(SOFA.SOFAnode.Made.TIR.AbsoluteName abs) throws java.rmi.RemoteException {
    StringBuffer sb = new StringBuffer();
    for (int i=0;i<abs.size()-1;i++) {
      sb.append(abs.elementAt(i));
      if (i < abs.size()-2)
        sb.append(".");
    }
    return sb.toString();
  }

  /** From CDL absolute name creates path. 
    *
    * @param abs         absolute name
    * @param shortName   short name (generated name can be different from abs name (reserved names))
    * @param pre         prepend to short name
    * @param post        append to short name
    * @param extension   if add ".java" extension
    */
  public static String absToPath(SOFA.SOFAnode.Made.TIR.AbsoluteName abs, String shortName, String pre, String post, boolean extension) throws java.rmi.RemoteException {
    StringBuffer sb = new StringBuffer();
    for (int i=0;i<abs.size()-1;i++) {
      sb.append(abs.elementAt(i));
//      if (i < abs.size()-1)
        sb.append(File.separator);
    }
    sb.append(pre);
    sb.append(shortName);
    sb.append(post);
    if (extension)
      sb.append(".java");
    return sb.toString();
  }

  /** From CDL absolute name of architecture creates path (without provider). 
    *
    * @param abs         absolute name
    * @param shortName   short name (generated name can be different from abs name (reserved names))
    * @param extension   if add ".java" extension
    */
  public static String archAbsToPath(SOFA.SOFAnode.Made.TIR.AbsoluteName abs, String shortName, boolean extension) throws java.rmi.RemoteException {
    StringBuffer sb = new StringBuffer();
    for (int i=1;i<abs.size()-1;i++) {
      sb.append(abs.elementAt(i));
//      if (i < abs.size()-1)
        sb.append(File.separator);
    }
    sb.append(shortName);
    if (extension)
      sb.append(".java");
    return sb.toString();
  }

  /** From CDL absolute name of architecture creates package. 
    *
    * @param abs         absolute name
    */
  public static String archAbsToPkg(SOFA.SOFAnode.Made.TIR.AbsoluteName abs) throws java.rmi.RemoteException {
    StringBuffer sb = new StringBuffer();
    for (int i=1;i<abs.size()-1;i++) {
      sb.append(abs.elementAt(i));
      if (i < abs.size()-2)
        sb.append(".");
    }
    return sb.toString();
  }

  /*
  public static void main(String[] argv) throws Exception {
    CGFileAccess fa = new CGFileAccess("/home/hnetynka/bla", true);
    CGFileWriter fw = fa.newCGWriter("cz/cuni/sofa/Pokus.java");
    fw.iwriteln("111111");
    fw.incIndent();
    fw.iwriteln("222222");
    fw.incIndent();
    fw.iwriteln("3333333");
    fw.decIndent();
    fw.iwriteln("222222");
    fw.iwriteln("222222");
    fw.decIndent();
    fw.iwriteln("111111");
    fw.close();
    System.out.println("OK");
  }  */

}
