package SOFA.SOFAnode.Made.TIR.Access.Browser;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Naming;
import java.rmi.RemoteException;

/** Print contents of repository as tree */
public class Main {
  public static void printKind(DefinitionKind dk) throws RemoteException {
    switch (dk.value()) {
      case DefinitionKind.dk_Attribute : System.out.print("Attribute"); break;
      case DefinitionKind.dk_Constant : System.out.print("Constant"); break;
      case DefinitionKind.dk_Exception : System.out.print("Exception"); break;
      case DefinitionKind.dk_Interface : System.out.print("Interface"); break;
      case DefinitionKind.dk_Module : System.out.print("Module"); break;
      case DefinitionKind.dk_Operation : System.out.print("Operation"); break;
      case DefinitionKind.dk_Typedef : System.out.print("Typedef"); break;
      case DefinitionKind.dk_Struct : System.out.print("Struct"); break;
      case DefinitionKind.dk_Union : System.out.print("Union"); break;
      case DefinitionKind.dk_Enum : System.out.print("Enum"); break;
      case DefinitionKind.dk_Primitive : System.out.print("Primitive"); break;
      case DefinitionKind.dk_String : System.out.print("String"); break;
      case DefinitionKind.dk_Sequence : System.out.print("Sequence"); break;
      case DefinitionKind.dk_Array : System.out.print("Array"); break;
      case DefinitionKind.dk_Repository : System.out.print("Repository"); break;
//      case DefinitionKind.dk_Template : System.out.print("Template"); break;
      case DefinitionKind.dk_Frame : System.out.print("Frame"); break;
      case DefinitionKind.dk_Architecture : System.out.print("Architecture"); break;
      case DefinitionKind.dk_Provides : System.out.print("Provides"); break;
      case DefinitionKind.dk_Requires : System.out.print("Requires"); break;
      case DefinitionKind.dk_Property : System.out.print("Property"); break;
//      case DefinitionKind.dk_ImplementationVer : System.out.print("ImplementationVer"); break;
      case DefinitionKind.dk_Implements : System.out.print("Implements"); break;
      case DefinitionKind.dk_Inst : System.out.print("Inst"); break;
      case DefinitionKind.dk_Bind : System.out.print("Bind"); break;
      case DefinitionKind.dk_Protocol : System.out.print("Protocol"); break;
      case DefinitionKind.dk_StructMember : System.out.print("StructMember"); break;
      case DefinitionKind.dk_EnumMember : System.out.print("EnumMember"); break;
      case DefinitionKind.dk_UnionMember : System.out.print("UnionMember"); break;
      case DefinitionKind.dk_Expression : System.out.print("Expression"); break;
      case DefinitionKind.dk_ProtocolOper : System.out.print("ProtocolOper"); break;
      case DefinitionKind.dk_ValueGen : System.out.print("ValueGen"); break;
      case DefinitionKind.dk_ValueGenElem : System.out.print("ValueGenOper"); break;
      case DefinitionKind.dk_BindOper : System.out.print("BindOper"); break;
      case DefinitionKind.dk_Provider : System.out.print("Provider"); break;
      case DefinitionKind.dk_AModule : System.out.print("AModule"); break;
      case DefinitionKind.dk_Wstring : System.out.print("Wstring"); break;
      case DefinitionKind.dk_Fixed : System.out.print("Fixed"); break;
    }
  }

  public static boolean isContainer(DefinitionKind dk) throws RemoteException {
    int v = dk.value();
    if (v == DefinitionKind.dk_Module) return true;
    if (v == DefinitionKind.dk_AModule) return true;
    if (v == DefinitionKind.dk_Provider) return true;
    if (v == DefinitionKind.dk_Interface) return true;
    if (v == DefinitionKind.dk_Struct) return true;
    if (v == DefinitionKind.dk_Union) return true;
    if (v == DefinitionKind.dk_Architecture) return true;
    if (v == DefinitionKind.dk_Exception) return true;
    if (v == DefinitionKind.dk_Frame) return true;
    if (v == DefinitionKind.dk_CDLRepository) return true;
    return false;
  }

  public static void intend(int in) {
    for(int i=0;i<in;i++)
      System.out.print("|  ");
  }

  public static void scanModule(Container what, int ind) throws RemoteException, TIRExceptLock {
    Contained con[] = what.contents(null);
    DefinitionKind dk;
    Identification id;
    for (int i=0;i<con.length;i++) {
      intend(ind);
      System.out.print("+--");
      printKind(dk = con[i].get_def_kind());
      id = con[i].get_identification();
      System.out.println(" \""+id.name()+"\" "+id.version());
     
      if (con[i] instanceof FrameDef) {
        if (((FrameDef)con[i]).is_system()) {
          intend(ind);
          System.out.println("***System frame");
        }
      }

      if (con[i] instanceof ArchitectureDef) {
        if (((ArchitectureDef)con[i]).is_system()) {
          intend(ind);
          System.out.println("***System architecure");
        }
      }
      
      
      if (con[i] instanceof InterfaceDef) {
        if (((InterfaceDef)con[i]).protocol() != null) {
          intend(ind);
          System.out.println(((InterfaceDef)con[i]).protocol().to_text());
        }
        InterfaceDef base[] = ((InterfaceDef)con[i]).base_interfaces();
        intend(ind);
        System.out.println(base.length);
      }
      if (con[i] instanceof FrameDef) {
        if (((FrameDef)con[i]).protocol() != null) {
          intend(ind);
	  System.out.println(((FrameDef)con[i]).protocol().to_text());
        }
      }
      
      
      if (isContainer(dk))
        scanModule((Container)con[i], ind+1);
    }  
  }
  
  public static void main(String[] argv) {
    try {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    
      String rmiport = System.getProperty("sofa.rmiport","1099");
      String rmihost = System.getProperty("sofa.rmihost","localhost");
      Repository obj = (Repository) Naming.lookup("//"+rmihost+":"+rmiport+"/Repository");
      System.out.println("Listing of repository: (kind \"name\" version)");
      scanModule(obj,0);
    } catch (Exception e) { 
      System.out.println("Exception: " + e.getMessage()); 
      e.printStackTrace(); 
    }
  }
}
