/* $Id: ProtocolOperandDefImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class ProtocolOperandDefImpl extends UnicastRemoteObject implements ProtocolOperandDef, TIRImplObject, SProtocolOperDef {
  String[] op;
  ProtocolMode mod;
  ProtocolOperKind pok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  

  public ProtocolOperandDefImpl(String[] oper, ProtocolMode mode) throws RemoteException {
    mod = mode;
    pok = new ProtocolOperKindImpl(ProtocolOperKind.pok_operand);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ProtocolOper);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    op = new String [oper.length];
    for (int i=0;i<oper.length;i++) {
      op[i] = new String(oper[i]);
    }
  }

  public ProtocolOperandDefImpl() throws RemoteException {
    op = null;
    mod = null;
    pok = new ProtocolOperKindImpl(ProtocolOperKind.pok_operand);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ProtocolOper);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }

  public ProtocolOperKind get_po_kind() throws RemoteException {
    return pok;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public String[] operand() throws RemoteException {
    return op;
  }
  
  public ProtocolMode operand_mode() throws RemoteException {
    return mod;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(pok.value());
      ((TIRImplObject) mod).save(st);
      st.curOutFile.writeInt(op.length);
      for(int i=0;i<op.length;i++) {
        Storage.writeString(st.curOutFile, op[i]);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      mod = new ProtocolModeImpl();
      ((TIRImplObject) mod).load(st);
      int j = st.curInFile.readInt();
      op = new String [j];
      for(int i=0;i<j;i++) {
        op[i] = Storage.readString(st.curInFile);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    stKindImpl.toNormal();
  }

  public void doAbort(long workId) throws RemoteException {}

  public String to_text() throws RemoteException {
    StringBuffer sb = new StringBuffer();
    if (mod.value() == ProtocolMode.pm_none) { // interface protocol
      sb.append(op[0]);
    } else {  // frame protocol
      if (mod.value() == ProtocolMode.pm_in)
        sb.append("?");
      else
        sb.append("!");
      sb.append(op[0]);
      for(int i=1;i<op.length;i++) {
        sb.append(".");
        sb.append(op[i]);
      }
    }
    return sb.toString();
  }
}
