/* $Id: RequireDefImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class RequireDefImpl extends ContainedImpl implements RequireDef, TIRImplObject {
  CDLType req;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;

  RequireDefImpl normal;
  long workId;

  public RequireDefImpl(Identification ident, Container in, Repository inrep, CDLType r, long workId) throws RemoteException {
    super(ident,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Requires);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    req = r;
    normal = null; 
    this.workId = workId;
  }

  public RequireDefImpl(Container in, Repository inrep) throws RemoteException {
    super(null,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Requires);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
    req = null;
    normal = null;
    this.workId = -1L;
  }

  public RequireDefImpl(RequireDefImpl a, Container in, Repository inrep, long workId) throws RemoteException {
    super(a.id,in,inrep);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_Requires);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
    req = a.req;
    normal = a;
    this.workId = workId;
  }

  public CDLType require() throws RemoteException {
    return req;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void tag(String t) throws RemoteException, TIRExceptCreate, TIRExceptLock {
    if (stKindImpl.value()!=StateKind.sk_work)
      throw new TIRExceptCreate("you can call create method on work object");
    super.tag(t);
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i;
      ((IdentificationImpl) id).save(st.curOutFile);
      Storage.writeCDLType(st, req);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int i;
      id = new IdentificationImpl();
      ((IdentificationImpl) id).load(st.curInFile);
      req = Storage.readCDLType(st);
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in file "+st.currentFile+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    try {
    
      Container con;
      Contained[] rt;
      Contained ret;
      IdentificationImpl id;
      AbsoluteName nm;
      boolean found;
      if (req.get_def_kind().value() == DefinitionKind.dk_none) {   // is tp NotYetInRep?
        nm = ((NotYetInRep) req).nm;
  	con = r;  // begin search from repository
        for (int i=0;i<nm.size()-1;i++) {  // last element is name of object
  	  id = new IdentificationImpl(nm.elementAt(0), "::"+nm.elementAt(i),"");
          rt = ((SContainer)con).slookup_name(id);
  	  if (rt.length==0) throw new TIRExceptStorage("Can't find type " + nm.name());
            if (rt.length==1) {
              con = (Container) rt[0];
  	  } else { 
  	    found = false;
            id.version(((NotYetInRep) req).id.version());
            int k = 0;
  	    for (k=0;k<rt.length;k++) {
  	      if (id.is_short_equal(rt[k].get_identification())) {
  	        found = true;
  		break;  // -- break for
  	      }
  	    } // -- end of for
  	    if (!found) throw new TIRExceptStorage("Can't find type " + nm.name());
  	    con = (Container) rt[k];
  	  }
  	}
  	if ((ret = ((SContainer)con).slookup(((NotYetInRep) req).id))==null)
  	  throw new TIRExceptStorage("Can't find type " + nm.name());
  	req = (CDLType) ret;
	if (req.get_def_kind().value()==DefinitionKind.dk_Array ||
	    req.get_def_kind().value()==DefinitionKind.dk_Sequence)
	  ((TIRImplObject) req).postLoad(r);  
      }

    } catch (TIRExceptLock e) {;}
  }

  public boolean isNew() {
    return ((stKindImpl.value()==StateKind.sk_work) && normal==null);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_normal)
      return;
    if (isNew()) {  
      stKindImpl.toNormal();
      switch (req.get_def_kind().value()) {
        case DefinitionKind.dk_Array:
        case DefinitionKind.dk_Sequence:
        case DefinitionKind.dk_String:
        case DefinitionKind.dk_Wstring:
        case DefinitionKind.dk_Fixed:
          ((TIRImplObject)req).doCommit(in,rep);
          break;
        default:
          Identification idl = ((Contained)req).get_identification();
          req = new NotYetInRep(idl.lang_absolute_name().name(), idl.name(), idl.version());
          break;
      }  
    } else
      normal = null;
  }

  public void doAbort(long workId) throws RemoteException {}
}
