/* $Id: ValueGenElementImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class ValueGenElementImpl extends UnicastRemoteObject implements ValueGenElement, TIRImplObject {
  ExprOperDef op1;
  ExprOperDef op2;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl; 

  public ValueGenElementImpl(ExprOperDef oper1, ExprOperDef oper2) throws RemoteException {
    op1 = oper1;
    op2 = oper2;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ValueGenElem);
    stKindImpl = new StateKindImpl(StateKind.sk_work);
  }

  public ValueGenElementImpl() throws RemoteException {
    op1 = null;
    op2 = null;
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ValueGenElem);
    stKindImpl = new StateKindImpl(StateKind.sk_normal);
  }

  public ExprOperDef expr1() throws RemoteException {
    return op1;
  }
  
  public ExprOperDef expr2() throws RemoteException {
    return op2;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      Storage.writeExprOperDef(st, op1);
      if (op2 !=null) {
        st.curOutFile.writeInt(1);
        Storage.writeExprOperDef(st, op2);
      } else {
        st.curOutFile.writeInt(0);
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in file \""+st.currentFile+"\".");
    }
  }

  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      op1 = Storage.readExprOperDef(st);
      if (st.curInFile.readInt()==1)
        op2 = Storage.readExprOperDef(st);
      else
        op2 = null;
    } catch (IOException e) {
      throw new TIRExceptStorage("Access error in file \""+st.currentFile+"\".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    ((TIRImplObject) op1).postLoad(r);
    if (op2 != null)
      ((TIRImplObject) op2).postLoad(r);
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      ((TIRImplObject)op1).doCommit(in,rep);
      if (op2 != null)
        ((TIRImplObject)op2).doCommit(in,rep);
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
