/* $Id: ValueGenImpl.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Made.TIR.Impl;
import SOFA.SOFAnode.Made.TIR.*;
import java.rmi.Remote;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;
import java.io.*;

public class ValueGenImpl extends UnicastRemoteObject implements ValueGen, TIRImplObject {
  ExprOperKindImpl eok;
  DefinitionKindImpl defKindImpl;
  StateKindImpl stKindImpl;
  int numOfItems;
  LiItem firstEl;
  LiItem lastEl;

  public ValueGenImpl(int state) throws RemoteException {
    eok = new ExprOperKindImpl(ExprOperKind.eok_valuegen);
    defKindImpl = new DefinitionKindImpl(DefinitionKind.dk_ValueGen);
    stKindImpl = new StateKindImpl(state);
    firstEl = null;
    lastEl = null;
  }

  public ValueGenElement[] elements() throws RemoteException {
    ValueGenElement[] ret = new ValueGenElement [numOfItems];
    LiItem akt = firstEl;
    for(int i=0;i<numOfItems;i++) {
      ret[i] = (ValueGenElement) akt.obj;
      akt = akt.next;
    }
    return ret;
  }
  
  public ValueGenElement create_element(ExprOperDef expr1, ExprOperDef expr2) throws RemoteException, TIRExceptCreate {
    if (stKindImpl.value() == StateKind.sk_normal)
      throw new TIRExceptCreate("You can call create method on work object only");
    ValueGenElement ret = new ValueGenElementImpl(expr1, expr2);  
    addListItem(ret);
    return ret;
  }

  /* from interface TIRObject */
  public DefinitionKind get_def_kind() throws RemoteException {
    return (DefinitionKind) defKindImpl;
  }

  /* from interface TIRObject */
  public StateKind get_state() throws RemoteException {
    return (StateKind) stKindImpl;
  }

  public ExprOperKind get_eok_kind() throws RemoteException {
    return (ExprOperKind) eok;
  }
  
  public ExprOperDef calculate() throws RemoteException, TIRExceptCalculate {
    throw new TIRExceptCalculate("Can't calculate value generator.");
  }

  public void addListItem(TIRObject what) {
    LiItem item = new LiItem(what);
    if (firstEl == null) {
      firstEl = lastEl = item;
    } else {
      LiItem akt = firstEl;
      while (akt.next != null) akt = akt.next;
      akt.next = item;
      item.prev = akt;
      lastEl = item;
    }
    numOfItems++;
  }

  public void save(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      st.curOutFile.writeInt(eok.value());
      st.curOutFile.writeInt(numOfItems);
      LiItem akt = firstEl;
      for(int i=0;i<numOfItems;i++) {
        ((TIRImplObject) akt.obj).save(st);
	akt = akt.next;
      }
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't write in directory "+st.current+".");
    }
  }
  
  public void load(Storage st) throws RemoteException, TIRExceptStorage {
    try {
      int num = st.curInFile.readInt();
      TIRImplObject obj;
      for(int i=0;i<num;i++) {
        obj = new ValueGenElementImpl();
        obj.load(st);
	addListItem((TIRObject) obj);
      }     
    } catch (IOException e) {
      throw new TIRExceptStorage("Can't read in directory "+st.current+".");
    }
  }

  public void postLoad(RepositoryImpl r) throws RemoteException, TIRExceptStorage {
    LiItem akt = firstEl;
    for (int i =0;i<numOfItems;i++) {
      ((TIRImplObject) akt.obj).postLoad(r);
      akt = akt.next;
    }  
  }

  public boolean isNew() {
    return (stKindImpl.value()==StateKind.sk_work);
  }

  public void canCommit() throws RemoteException, TIRExceptCommit {;}

  public void doCommit(Container in, Repository rep) throws RemoteException {
    if (stKindImpl.value()==StateKind.sk_work) {
      stKindImpl.toNormal();
      LiItem akt = firstEl;
      for (int i=0;i<numOfItems;i++) {
        ((TIRImplObject)akt.obj).doCommit(in,rep);
        akt = akt.next;     
      }
    }
  }

  public void doAbort(long workId) throws RemoteException {}
}
