/* $Id: ComponentAccess.java,v 1.1 2004/03/22 19:31:49 hnetynka Exp $ */
package SOFA.SOFAnode.Run.Deployment;

import java.net.MalformedURLException;
import java.net.URL;
import java.net.InetAddress;
import java.net.UnknownHostException;

import SOFA.SOFAnode.Run.DeplDockRegistry.*;

import org.omg.CosNaming.*;

/** Allows acess to SOFA components from ordinary applications. 
  * 
  * @author Petr Hnetynka
  */
public class ComponentAccess {
  
  private SOFA.Component.ThreadIDRegistry thrReg;
  private SOFA.Util.CORBAAccess ca;
  
  /** Reference to deployment dock registry. */
  public DeplDockRegistry ddr;
  
  /** Constructor. 
    * 
    * @throws ComponentAccessException cannot initialize all necessary objects
    */
  public ComponentAccess() throws ComponentAccessException {
    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new java.rmi.RMISecurityManager());
    }

    String sofaNodeName = System.getProperty("sofa.nodename","");
    if (sofaNodeName.compareTo("")==0) {
      try {
        sofaNodeName = InetAddress.getLocalHost().getHostName();
      } catch (UnknownHostException e) { 
        throw new ComponentAccessException("Can't get host name. Set name to property \"sofa.nodename\"", e);
      }
    }

    System.setProperty("sofa.nodename",sofaNodeName);
    thrReg = new SOFA.Component.ThreadIDRegistryImpl(sofaNodeName);

    ca = new SOFA.Util.CORBAAccess();
    NameComponent nc = new NameComponent("DeplDockRegistry", "");
    NameComponent path[] = {nc};
    try {
      ddr = DeplDockRegistryHelper.narrow(SOFA.Util.CORBAAccess.ncRef.resolve(path));
    } catch (Exception e) {
      throw new ComponentAccessException("Can't get DeplDock registry.", e);
    }
  }

  /** Corrects connector part in order to be used in non component application. It assumes that connector
    * is obtained via component introspection (Component.getFcInterface[s]).
    * 
    * @param rb connector part
    * @return corrected connector part
    */
  public SOFA.Connector.RoleBase correctRoleBase(SOFA.Connector.RoleBase rb) {
    rb.setThrIDRegistry(thrReg);
    return rb;
  }

  /** Returns deployment dock. 
    * 
    * @param name name of the dock
    * @return deployment dock
    * @throws ComponentAccessException 
    */
  public SOFA.SOFAnode.Run.Deployment.DeplDock getDock(String name) throws ComponentAccessException {
    byte[] serRef = null;
    try {
      serRef = ddr.lookup(name);
    } catch (SOFA.SOFAnode.Run.DeplDockRegistry.NotFoundException e) {
      throw new ComponentAccessException("No deployment dock with name \""+name+"\".");
    }

    SOFA.Connector.Reference ddRef = SOFA.SOFAnode.Run.Deployment.DeplDockImpl.bytesToSofaReference(serRef);

    SOFA.SOFAnode.Run.Deployment.DeplDock ddock = null;
    try {
      ddock = (SOFA.SOFAnode.Run.Deployment.DeplDock) SOFA.Connector.Boot.DeplDockConnector.createClt(ddRef);
    } catch (SOFA.Connector.ConnectorException e) {
      throw new ComponentAccessException("Can't create connector to deployment dock.");
    }
    return ddock;
  }


}
