/**
 * TR2InOutConnector.java is a part of the SOFA project.
 * This file was created by pepan on 1.4.2003.
 */
package SOFA.SOFAnode.TR.Connector;

import SOFA.Connector.*;
import SOFA.SOFAnode.TR.TR2InOut;
import SOFA.Util.VMProperties;

/** Contains methods for creating a connector between TR and InOut interfaces.
 *
 * @author Petr Panuska
 */
public class TR2InOutConnector {

  /**
   * Creates the server side of a connector between {@link SOFA.SOFAnode.TR.Impl.TRImpl TR}
   * and {@link SOFA.SOFAnode.InOut.Impl.InOutImpl InOut}. The connector is defined in
   * the {@link TR2InOut} interface.
   * @param tr TR which is going to be connectable.
   * @return the server side of the connector.
   */
  public static RoleBase createSrv (TR2InOut tr) throws ConnectorException {
    RoleBase role = new TR2InOutSRole();
    SOFA.Connector.ECG.DeploymentDescriptor cDD = new SOFA.Connector.ECG.DeploymentDescriptor();
    cDD.unit = "Server";
    role.setDeploymentDescriptor(cDD);

    Linkable skelRMI;
    String rmiName = "";
    String rmihost = System.getProperty(VMProperties.RMI_HOST, "localhost");
    String rmiport = System.getProperty(VMProperties.RMI_PORT, "1099");
    String trName = System.getProperty(VMProperties.TR_NAME, "localTR");
    trName = "/SOFA/SOFAnode/TR/Connector/TR2InOutRMISkel/" + trName;
    try {
      rmiName = "//" + rmihost + ":" + rmiport + trName;
      skelRMI = new TR2InOutRMISkel(trName);
    } catch (Exception e) {
      throw new ConnectorException("Can't instantiate TR2InOutRMISkel." + rmiName + "\n", e);
    }
    skelRMI.link(role);

    // ???
    Reference ref = new Reference("SOFA.Connector.ECG.SGenerator.Connectors.DeplDockConnector",
      new TaggedProfile[]{new SOFA.Connector.ECG.Profiles.LocalProfile(role), new SOFA.Connector.ECG.Profiles.RMIProfile(rmiName)});
    role.setSOFAReference(ref);
    role.link(tr);
    return role;
  }

  /**
   * Creates the client side of a connector between {@link SOFA.SOFAnode.TR.Impl.TRImpl TR}
   * and {@link SOFA.SOFAnode.InOut.Impl.InOutImpl InOut}. The connector is defined in
   * the {@link TR2InOut} interface.
   * @param ref the reference to TR.
   * @return the client side of the connector.
   */
  public static RoleBase createClt (Reference ref) throws ConnectorException {
    RoleBase role = new SOFA.SOFAnode.TR.Connector.TR2InOutCRole();
    try {
      role.link(ref);
    } catch (LinkException e) {
      throw new ConnectorException("Can't link connector to TR.\n" + ref + "\n", e);
    }
    return role;
  }
}
