/**
 * BundleEntry.java is a part of the SOFA project.
 * This file was created by pepan on 21.3.2003.
 */
package SOFA.SOFAnode.TR.Impl;

import java.io.File;
import java.util.jar.JarEntry;

/**
 * One bundle entry (a component) of bundle input/output streams. It's used when
 * putting a component to the stream or getting a component from it. Its use is
 * similar to {@link JarEntry}.
 * @author Petr Panuska
 */
class BundleEntry extends JarEntry {

  /**
   * All filles creating one component.
   */
  private ComponentFiles files;

  /**
   * Creates an entry with a given name. The name should be unique.
   * @param name the name of this entry in a JAR file.
   */
  BundleEntry (String name) {
    super(name);
    files = new ComponentFiles();
  }

  /**
   * Creates an entry with a given name and assigns the component files.
   * The name should be unique.
   * @param name the name of this entry in a JAR file.
   * @param files a set of files creating the component being described by this
   * entry.
   */
  BundleEntry (String name, ComponentFiles files) {
    super(name);
    this.files = files;
  }

  /**
   * Adds a file under a given unique name. The file should not be a directory.
   * @param file the file being added.
   * @param name the unique name.
   */
  void addFile (File file, String name) {
    files.addFile(file, name);
  }

  /**
   * Gets files being a part of this entry.
   * @return all files creating a component (which this entry describes).
   */
  ComponentFiles getFiles () {
    return files;
  }
}
