/**
 * Main.java is a part of the SOFA project.
 * This file was created by pepan on 20.3.2003.
 */
package SOFA.SOFAnode.TR.Impl;

import SOFA.Connector.Reference;
import SOFA.SOFAnode.InOut.Impl.Browser;
import SOFA.SOFAnode.TR.Connector.TR2InOutConnector;
import SOFA.Util.VMProperties;

import java.net.InetAddress;
import java.net.UnknownHostException;
import java.rmi.RMISecurityManager;

/**
 * Runs TR, InOut and possibly the graphical client.
 * @author Petr Panuska
 */
public class Main {

  /**
   * Runs TR, InOut and possibly the client. The client is started only in the case when
   * a parameter on the command line is provided.
   * @param argv the single argument is a path to a client configuration file.
   * The configuration file is a a text file listing all known (remote/local) SOFA nodes.
   * Every line consists of a key and a value (key=value[new line]), the key is a name
   * of the SOFA node as it appears in the client, the value is a string form of
   * a connector pointing to the InOut part (InOut2Client connector) of the SOFA node.
   */
  public static void main (String[] argv) {

    if (System.getSecurityManager() == null) {
      System.setSecurityManager(new RMISecurityManager());
    }
    String trDir = System.getProperty(VMProperties.TR_DIR, null);
    if (trDir == null) {
      System.out.println("The property '" + VMProperties.TR_DIR + "' has not been specified.");
      System.exit(1);
    }
    try {
      String sofaNodeName = System.getProperty(VMProperties.NODE_NAME);

      if (sofaNodeName == null) {
        try {
          sofaNodeName = InetAddress.getLocalHost().getHostName();
        } catch (UnknownHostException e) {
          System.err.println("Can't get the host name.\nSet the name to the property '" + VMProperties.NODE_NAME + "' when TR is being started.");
          System.exit(1);
        }
      }

      System.setProperty(VMProperties.NODE_NAME, sofaNodeName);

      TRImpl tr = new TRImpl(trDir);

      System.out.print("Registering TR....");
      Reference refTR = TR2InOutConnector.createSrv(tr).getSOFAReference();
      System.out.println("OK");
      System.out.println(refTR);

      SOFA.SOFAnode.InOut.Impl.Main.runInOut(refTR, null); // run InOut server in the same address space

    } catch (Exception e) {
      System.out.println("Exception " + e.getMessage());
      e.printStackTrace();
    }
    if (argv.length == 1) {
      new Browser(argv[0]);
    } else
      System.out.println("Running without client!!\n");
  }
}
