/* $Id: ActionTokenArray.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */

package SOFA.SOFAnode.Util;

import java.io.*;
import java.util.*;

/**
 * The class for collection of action tokens with support for
 * lookup by action token with the same event name.
 *
 * @author Stanislav Visnovsky
 * @version 1.0.0
 *
 * @see ActionToken
*/

public class ActionTokenArray {

    public boolean tokens[];

    public int size;

/**
 * Simple constructor of an empty array. Its size is determined from number 
 * of all action tokens in EdgeFactory table.
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
 * @see EdgeFactory#ActionTokens
 */
    public ActionTokenArray() {
        tokens = new boolean[ EdgeFactory.getActionTokensNum() ];
	size = EdgeFactory.getActionTokensNum();
    }

/**
 * Simple copy constructor of a collection from hashtable. This requires that 
 * EdgeFactory.ActionTokens contains all action tokens in hashtable.
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
 * @see EdgeFactory.ActionTokens
 */
    public ActionTokenArray( Hashtable h ) {
	this();
	Debug.println(3, "ActionTokenArray: This is hashtable copy constructor");
	for( Iterator i = h.values().iterator(); i.hasNext(); ) {
	    ActionToken t = (ActionToken)i.next() ;
	    if( t.isEvent() ) {
		int j = EdgeFactory.getActionTokenIdx( new ActionToken( "?"+t.name) );
		this.tokens[j] = true;
//		this.tokens[ EdgeFactory.inverseActionTokens[j]] = true;
	    } else tokens[ EdgeFactory.getActionTokenIdx( t )] = true;
	}
	Debug.println(3, "ActionTokenArray: Finished hashtable copy constructor");
    }
  
/**
 * Simple copy constructor of a collection from other array
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
 */
    public ActionTokenArray( ActionTokenArray h ) {
	this();
	for( int i = 0 ; i < h.size ; i ++ ) 
	    tokens[i] = h.tokens[i];
    }
  
/**
 * Constructor of an collection from an array of strings representing action tokens or events.
 * If an element does not have an action suffix, it creates two action tokens correponding to
 * the request and the response.
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
 */
    public ActionTokenArray( String[] tokens ) {

	this.tokens = new boolean[ EdgeFactory.getActionTokensNum()];
	size = EdgeFactory.getActionTokensNum();

	// fill the collection
        for( int i=0; i<tokens.length ; i++ ) {
            ActionToken a = new ActionToken(tokens[i]);
            if( a.isSimpleCall() ) {
		int j = EdgeFactory.getActionTokenIdx( new ActionToken(tokens[i]+"^") );
		this.tokens[j] = true;
		j = EdgeFactory.getActionTokenIdx( new ActionToken(tokens[i]+"$").toInverseActionToken() );
		this.tokens[ j ] = true;
	    } else this.tokens[EdgeFactory.getActionTokenIdx( new ActionToken( tokens[i]))] = true;
	}
    }

/**
 * Method for membership test for the action token with the same
 * event as t has.
 *
 * @author	Stanislav Visnovsky
 * @version	2.0.0
 *
 * @return		true if collection contains corresponding action token
 * @param t		t with event to lookup
 *
 * @see	ActionToken#toInverseActionToken
 * @see ActionToken#toTauActionToken
 */
    public boolean containsWithEventName( int tran ) {
	if( tran == -1 ) return false;
        int inverse = EdgeFactory.inverseActionTokens[tran];
	if( inverse == -1 ) inverse = tran;
        int tau = EdgeFactory.tauActionTokens[tran];
	if( tau == -1 ) tau = tran;

	return ( tokens[ tran ] || tokens[ inverse ] || tokens[tau] );
    }

/**
 * Transforms the collection into a comma-separated list of action tokens.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 *
 * @return		generated list
 *
 */
    public String toList() {
        String l = "";
	for( int i = 0 ; i<size ; i ++ ) 
	    if( tokens[i] ) 
		l = l + "," + EdgeFactory.ActionTokens[i].name;

        if( l != "" ) l = l.substring(1); // eliminate leading ,
        Debug.println( 3, "List of tokens: "+l );
        return l;
    }
}