/* $Id: GenFactory.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */
package SOFA.SOFAnode.Util.ConnectorProtocolGen;

import SOFA.SOFAnode.Made.CDL.*;
import org.w3c.dom.*;
import java.lang.reflect.*;

/** Class providing public methods for working with connector protocol generators. 
  * @author Tomas Bures
  */
public final class GenFactory {
  
	protected static java.util.HashMap generators=new java.util.HashMap();

  /** Initializes the GenFactory.
    * @throws CannotInitException thrown when it is not possible to initialize the factory. (e.g. bad configuration file) 
    */
  public static void init() throws CannotInitException {
	
    generators.put("CSProcCall","SOFA.SOFAnode.Util.ConnectorProtocolGen.CSProcCall");

    String confFile=System.getProperty("SOFA.SOFAnode.Util.ConnectorProtocolGen.GeneratorsConfig");

    if (confFile!=null) {
      try {
        javax.xml.parsers.DocumentBuilderFactory documentBuilderFactory;
        javax.xml.parsers.DocumentBuilder documentBuilder;

        documentBuilderFactory=javax.xml.parsers.DocumentBuilderFactory.newInstance();
        documentBuilder=documentBuilderFactory.newDocumentBuilder();
		
        Document configDoc=documentBuilder.parse(new java.io.File(confFile));

        Element gens=configDoc.getDocumentElement();

        System.out.println("Loading connector protocol generators specification:");

        Node gen=gens.getFirstChild();
        while (gen!=null) {
          if (gen.getNodeType()==Node.ELEMENT_NODE && gen.getNodeName().equals("generator")) {
            Element elg=(Element)gen;

            if (!(elg.hasAttribute("type") && elg.hasAttribute("class"))) {
              throw new CannotInitException("Missing attributes in connector declaration.");
            }
            generators.put(elg.getAttribute("type"),elg.getAttribute("class"));
          }
          gen=gen.getNextSibling();
        }
      } catch (Exception e) {
        throw new CannotInitException("Can't parse configuration.",e);
      }
    }
  }

  /** Combines bindings which belong to one connector.
    * @param detectedConnectors  list of so far identified connectors
    * @param componentBinding  binding of two component interfaces (via keyword "using")
    * @throws BadBindException  thrown if the binding is invalid (e.g. interface is connected using different connector types)
    */
  public static void addDetectedConnector(String[] connectorOperations, DetectedConnectors detectedConnectors, CompBind componentBinding) throws GeneratorNotFoundException, BadBindException {
    String connectorType=componentBinding.using;
    String className=(String)generators.get(connectorType);

    if (className==null) {
      throw new GeneratorNotFoundException("Connector type '"+connectorType+"' in not registered in GenFactory.");
    }
    className+="Binding";

    try {
      Class genClass=Class.forName(className);
      Method mt=genClass.getMethod("addDetectedConnector",new Class[] {String[].class, DetectedConnectors.class, CompBind.class});
      mt.invoke(null,new Object[] {connectorOperations,detectedConnectors,componentBinding});
    } catch (InvocationTargetException e) {
      throw (BadBindException)e.getCause();
    } catch (Exception e) {
      throw new GeneratorNotFoundException("Cannot call static method 'addDetectedConnector' in class '"+className+"' for connector type '"+connectorType+"'.",e);
    }
  }

  /** Creates a new connector protocol generator according to connector type
    * @param connectorType  type of connector (e.g. "CSProcCall")
    * @return  created connector protocol generator
    * @throws GeneratorNotFoundException  thrown if the specified connector type is not registered 
    */
  public static ConnectorProtocolGen getConnectorProtocolGen(String connectorType) throws GeneratorNotFoundException {
    String className=(String)generators.get(connectorType);
    if (className==null) {
      throw new GeneratorNotFoundException("Connector type '"+connectorType+"' in not registered in GenFactory.");
    }

    try {
      Class genClass=Class.forName(className);
      return (ConnectorProtocolGen)genClass.newInstance();
    } catch (Exception e) {
      throw new GeneratorNotFoundException("Cannot intantiate class '"+className+"' for connector type '"+connectorType+"'.",e);
    }
  }
}

