/* $Id: Debug.java,v 1.1.1.1 2003/02/11 16:19:41 bures Exp $ */

package SOFA.SOFAnode.Util;

/**
 * The debug enabling class. If the {@link Debug#debug} is set to number higher
 * than 0, all methods prints the arguments, otherwise the output is skipped.
 * The set of methods allowing specification of a debug level to which the message
 * is intended is also provided. Supposed use of debug level:
 * 0 - no debug info
 * 1 - only high level debug about progress
 * 2 - progress of complicated methods
 * 3 - all debug info
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
*/
class Debug {
/**
 * Indication, if the debugging stuff should be send to the standard output
*/
    public static int debug = 0;

/**
 * Print a string to the standard output and the new line.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @param a	a string to print
*/
    public static void println( String a ) {
	if( debug>0 ) System.out.println( a );
    }

    public static void println( int level, String a ) {
	if( debug>=level ) System.out.println( a );
    }

/**
 * Print a new line to the standard output.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
*/
    public static void println() {
	if( debug>0 ) System.out.println();
    }

    public static void println(int level) {
	if( debug>=level ) System.out.println();
    }

/**
 * Print an integer to the standard output and the new line.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @param a	an integer to print
 * @param level debug level the message should  be printed at
*/
    public static void println( int level, int a ) {
	if( debug>=level ) System.out.println( a );
    }

/**
 * Print a string to the standard output.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @param a	a string to print
*/
    public static void print( String a ) {
	if( debug>0 ) System.out.print( a );
    }

    public static void print( int level, String a ) {
	if( debug>=level ) System.out.print( a );
    }

/**
 * Print an integer to the standard output.
 *
 * @author	Stanislav Visnovsky
 * @version	1.0.0
 * @param a	an integer to print
 * @param level debug level the message should be printed at
*/
    public static void print( int level, int a ) {
	if( debug>=level ) System.out.print( a );
    }
}
